
function UpdatePlayerData(includeItemInfo)
	local player = Turbine.Gameplay.LocalPlayer.GetInstance();

	-- Get character data for the local player
	if not LocalPlayerCharacter then
		-- Must be first call, find the local player index
		localPlayerName = player:GetName();
		for i = 1, #CharacterList do
			if CharacterList[i].name == localPlayerName then
				LocalPlayerCharacter = i;
			end
		end

		-- No existing character data, create new
		if not LocalPlayerCharacter then
			local newCharacter = { name = localPlayerName };
			table.insert(CharacterList, newCharacter);

			-- Keep sorted
			local characterSorter = function(character1, character2)
				return character1.name < character2.name;
			end
			table.sort(CharacterList, characterSorter);

			-- Find new index
			for i = 1, #CharacterList do
				if CharacterList[i].name == localPlayerName then
					LocalPlayerCharacter = i;
				end
			end
		end
	end

	local character = CharacterList[LocalPlayerCharacter];

	local playerAttr = player:GetAttributes();
	character.vocation = playerAttr:GetVocation();
	character.professions = nil;

	if character.vocation then
		character.professions = {};

		for k,v in pairs(Turbine.Gameplay.Profession) do
			local professionInfo = playerAttr:GetProfessionInfo(v);

			if professionInfo then
				local professionData = { }
			
				professionData.proficiencyLevel = professionInfo:GetProficiencyLevel();
				professionData.masteryLevel = professionInfo:GetMasteryLevel();

				professionData.recipes = { }
				professionData.recipeHash = {};
				for i = 1, professionInfo:GetRecipeCount() do
					local recipe = professionInfo:GetRecipe(i);

					recipeData = {};
					recipeData.name = recipe:GetName();
					recipeData.category = recipe:GetCategoryName();
					recipeData.tier = recipe:GetTier();

					local itemInfo = recipe:GetResultItemInfo();
					if includeItemInfo then
						recipeData.resultItemInfo = itemInfo;
					end
					recipeData.resultIcon = itemInfo:GetIconImageID();
					recipeData.resultIconBack = itemInfo:GetBackgroundImageID();

					professionData.recipes[i] = recipeData;

					professionData.recipeHash[recipeData.name] = true;
				end

				character.professions[v] = professionData;
			end
		end
	end

end

function GetTierName(tier)
	for k, v in pairs(Turbine.Gameplay.CraftTier) do
		if v == tier then
			return k;
		end
	end
	return "Invalid tier";
end

function GetProfessionName(profession)
	for k, v in pairs(Turbine.Gameplay.Profession) do
		if v == profession then
			return k;
		end
	end
	return "Invalid profession";
end

function GetVocationName(vocation)
	for k, v in pairs(Turbine.Gameplay.Vocation) do
		if v == vocation then
			return k;
		end
	end
	return "Invalid vocation";
end

function CreateRecipeTree(recipes)
	local tree = {};

	for i = 1, #recipes do
		local recipe = recipes[i];

		local tierTree = tree[recipe.tier];
		if not tierTree then
			tierTree = {};
			tree[recipe.tier] = tierTree;
		end

		local categoryTree = tierTree[recipe.category];
		if not categoryTree then
			categoryTree = {};
			tierTree[recipe.category] = categoryTree;
		end

		table.insert(categoryTree, recipe);
	end

	local recipeSorter = function(recipe1, recipe2)
		return recipe1.name < recipe2.name;
	end

	-- sorting, add a sorted category list in each tier, and sort the recipe list directly
	for tier, tierTree in pairs(tree) do
		local categoryList = {};
		for category, categoryTree in pairs(tierTree) do
			table.insert(categoryList, category);
			table.sort(categoryTree, recipeSorter);
		end

		table.sort(categoryList);
		tierTree.categoryList = categoryList;
	end

	return tree;
end
